@@ -8,15 +8,16 @@
 use std::ffi::CStr;
 use std::str::FromStr;
 
+use ::log::warn;
 use env_logger::Builder;
 use libc::c_char;
+use pact_models::interaction::Interaction;
+use pact_models::pact::Pact;
+use pact_models::v4::pact::V4Pact;
 
 use models::message::Message;
-use pact_matching::{self as pm};
+use pact_matching as pm;
 pub use pact_matching::Mismatch;
-use pact_models::interaction::Interaction;
-use pact_models::v4::pact::V4Pact;
-use pact_models::pact::Pact;
 
 use crate::util::*;
 
@@ -42,15 +43,17 @@ pub extern "C" fn pactffi_version() -> *const c_char {
 ///
 /// # Safety
 ///
-/// Exported functions are inherently unsafe.
+/// log_env_var must be a valid NULL terminated UTF-8 string.
 #[no_mangle]
 pub unsafe extern fn pactffi_init(log_env_var: *const c_char) {
+    init_windows();
+
     let log_env_var = if !log_env_var.is_null() {
         let c_str = CStr::from_ptr(log_env_var);
         match c_str.to_str() {
             Ok(str) => str,
             Err(err) => {
-                ::log::warn!("Failed to parse the environment variable name as a UTF-8 string: {}", err);
+                warn!("Failed to parse the environment variable name as a UTF-8 string: {}", err);
                 "LOG_LEVEL"
             }
         }
@@ -63,6 +66,16 @@ pub unsafe extern fn pactffi_init(log_env_var: *const c_char) {
     builder.try_init().unwrap_or(());
 }
 
+#[cfg(windows)]
+fn init_windows() {
+    if let Err(err) = ansi_term::enable_ansi_support() {
+        warn!("Could not enable ANSI console support - {err}");
+    }
+}
+
+#[cfg(not(windows))]
+fn init_windows() { }
+
 /// Initialises logging, and sets the log level explicitly.
 ///
 /// # Safety
