@@ -1,13 +1,25 @@
 //! Handle interface to creating a verifier
 
 use std::sync::Arc;
+use itertools::Itertools;
 
 use log::debug;
-
 use pact_models::prelude::HttpAuth;
-use pact_verifier::{FilterInfo, NullRequestFilterExecutor, PactSource, ProviderInfo, VerificationOptions, verify_provider_async, ConsumerVersionSelector};
+use serde_json::Value;
+
+use pact_verifier::{
+  ConsumerVersionSelector,
+  FilterInfo,
+  NullRequestFilterExecutor,
+  PactSource,
+  ProviderInfo,
+  PublishOptions,
+  VerificationOptions,
+  verify_provider_async
+};
 use pact_verifier::callback_executors::HttpRequestProviderStateExecutor;
 use pact_verifier::metrics::VerificationMetrics;
+use pact_verifier::verification_result::VerificationExecutionResult;
 
 #[derive(Debug, Clone)]
 /// Wraps a Pact verifier
@@ -16,10 +28,13 @@ pub struct VerifierHandle {
   sources: Vec<PactSource>,
   filter: FilterInfo,
   state_change: Arc<HttpRequestProviderStateExecutor>,
-  options: VerificationOptions<NullRequestFilterExecutor>,
+  verification_options: VerificationOptions<NullRequestFilterExecutor>,
+  publish_options: Option<PublishOptions>,
   consumers: Vec<String>,
   /// Calling application name and version
-  calling_app: Option<(String, String)>
+  calling_app: Option<(String, String)>,
+  /// Output captured from the verifier
+  verifier_output: VerificationExecutionResult
 }
 
 impl VerifierHandle {
@@ -31,9 +46,11 @@ impl VerifierHandle {
       sources: Vec::new(),
       filter: FilterInfo::None,
       state_change: Arc::new(HttpRequestProviderStateExecutor::default()),
-      options: VerificationOptions::default(),
+      verification_options: VerificationOptions::default(),
+      publish_options: None,
       consumers: vec![],
-      calling_app: None
+      calling_app: None,
+      verifier_output: VerificationExecutionResult::new()
     }
   }
 
@@ -44,9 +61,11 @@ impl VerifierHandle {
       sources: Vec::new(),
       filter: FilterInfo::None,
       state_change: Arc::new(HttpRequestProviderStateExecutor::default()),
-      options: VerificationOptions::default(),
+      verification_options: VerificationOptions::default(),
+      publish_options: None,
       consumers: vec![],
-      calling_app: Some((calling_app_name.to_string(), calling_app_version.to_string()))
+      calling_app: Some((calling_app_name.to_string(), calling_app_version.to_string())),
+      verifier_output: VerificationExecutionResult::new()
     }
   }
 
@@ -126,7 +145,6 @@ impl VerifierHandle {
   pub fn add_pact_broker_source(
     &mut self,
     url: &str,
-    provider_name: &str,
     enable_pending: bool,
     include_wip_pacts_since: Option<String>,
     provider_tags: Vec<String>,
@@ -136,7 +154,7 @@ impl VerifierHandle {
   ) {
     if !auth.is_none() {
       self.sources.push(PactSource::BrokerWithDynamicConfiguration {
-        provider_name: provider_name.to_string(),
+        provider_name: self.provider.name.clone(),
         broker_url: url.to_string(),
         enable_pending,
         include_wip_pacts_since,
@@ -148,7 +166,7 @@ impl VerifierHandle {
       });
     } else {
       self.sources.push(PactSource::BrokerWithDynamicConfiguration {
-        provider_name: provider_name.to_string(),
+        provider_name: self.provider.name.clone(),
         broker_url: url.to_string(),
         enable_pending,
         include_wip_pacts_since,
@@ -175,26 +193,42 @@ impl VerifierHandle {
     })
   }
 
-  /// Update the verification options
+  /// Update options used when running a verification
+  /// 
+  /// # Args
+  /// 
+  /// - disable_ssl_verification - Disable SSL verification on all HTTPS requests
+  /// - request_timeout - Timeout for all requests to the provider
   pub fn update_verification_options(
     &mut self,
-    publish: bool,
-    provider_version: &str,
-    build_url: Option<String>,
-    provider_tags: Vec<String>,
     disable_ssl_verification: bool,
     request_timeout: u64
   ) {
-    self.options = VerificationOptions {
-      publish,
+    self.verification_options = VerificationOptions {
+      request_filter: None::<Arc<NullRequestFilterExecutor>>,
+      disable_ssl_verification,
+      request_timeout
+    }
+  }
+
+  /// Update the details used when publishing results
+  /// 
+  /// # Args
+  /// 
+  /// - `provider_version` - Version of the provider
+  pub fn update_publish_options(
+    &mut self,
+    provider_version: &str,
+    build_url: Option<String>,
+    provider_tags: Vec<String>,
+    provider_branch: Option<String>
+  ) {
+    self.publish_options = Some(PublishOptions {
       provider_version: Some(provider_version.to_string()),
       build_url,
-      request_filter: None::<Arc<NullRequestFilterExecutor>>,
       provider_tags,
-      disable_ssl_verification,
-      request_timeout,
-      .. VerificationOptions::default()
-    }
+      provider_branch,
+    })
   }
 
   /// Update the consumer filter
@@ -211,22 +245,25 @@ impl VerifierHandle {
   /// * 0 - verification was successful
   /// * 1 - verification was not successful
   /// * 2 - failed to run the verification
-  pub fn execute(&self) -> i32 {
+  ///
+  /// Anu captured output from the verification will be stored against this handle
+  pub fn execute(&mut self) -> i32 {
     for s in &self.sources {
-      debug!("Pact source to verify = {}", s);
+      debug!("Pact source to verify = {s}");
     };
 
     let (calling_app_name, calling_app_version) = self.calling_app.clone().unwrap_or_else(|| {
       ("pact_ffi".to_string(), env!("CARGO_PKG_VERSION").to_string())
     });
     let runtime = tokio::runtime::Runtime::new().unwrap();
-    runtime.block_on(async {
+    match runtime.block_on(async {
       verify_provider_async(
         self.provider.clone(),
         self.sources.clone(),
         self.filter.clone(),
         self.consumers.clone(),
-        self.options.clone(),
+        &self.verification_options,
+        self.publish_options.as_ref(),
         &self.state_change.clone(),
         Some(VerificationMetrics {
           test_framework: "pact_ffi".to_string(),
@@ -234,9 +271,24 @@ impl VerifierHandle {
           app_version: calling_app_version.clone()
         })
       ).await
-    })
-      .map(|result| if result { 0 } else { 2 })
-      .unwrap_or(2)
+    }) {
+      Ok(result) => {
+        self.verifier_output = result.clone();
+        if result.result { 0 } else { 1 }
+      }
+      Err(_) => 2
+    }
+  }
+
+  /// Return the captured standard output from the verification execution
+  pub fn output(&self) -> String {
+    self.verifier_output.output.iter().join("\n")
+  }
+
+  /// Return the verification results as a JSON document
+  pub fn json(&self) -> String {
+    let json: Value = (&self.verifier_output).into();
+    json.to_string()
   }
 }
 
