@@ -109,10 +109,10 @@ ffi_fn! {
   /// Returns NULL on error.
   fn pactffi_verifier_new_for_application(
     name: *const c_char,
-    scheme: *const c_char
+    version: *const c_char
   ) -> *mut handle::VerifierHandle {
     let name = if_null(name, "unknown");
-    let version = if_null(scheme, "unknown");
+    let version = if_null(version, "unknown");
     let handle = handle::VerifierHandle::new_for_application(name.as_str(), version.as_str());
     ptr::raw_to(handle)
   } {
@@ -211,9 +211,8 @@ ffi_fn! {
 }
 
 ffi_fn! {
-    /// Set the verification options for the Pact verifier.
+    /// Set the options used by the verifier when calling the provider
     ///
-    /// `publish` is a boolean value. Set it to greater than zero to turn the option on.
     /// `disable_ssl_verification` is a boolean value. Set it to greater than zero to turn the option on.
     ///
     /// # Safety
@@ -223,27 +222,12 @@ ffi_fn! {
     ///
     fn pactffi_verifier_set_verification_options(
       handle: *mut handle::VerifierHandle,
-      publish: c_uchar,
-      provider_version: *const c_char,
-      build_url: *const c_char,
       disable_ssl_verification: c_uchar,
-      request_timeout: c_ulong,
-      provider_tags: *const *const c_char,
-      provider_tags_len: c_ushort
+      request_timeout: c_ulong
     ) -> c_int {
       let handle = as_mut!(handle);
-      let provider_version = safe_str!(provider_version);
-      let build_url = if_null(build_url, "");
 
-      let build_url = if !build_url.is_empty() {
-        Some(build_url)
-      } else {
-        None
-      };
-
-      let tags = get_vector(provider_tags, provider_tags_len);
-
-      handle.update_verification_options(publish > 0, provider_version, build_url, tags, disable_ssl_verification > 0, request_timeout as u64);
+      handle.update_verification_options(disable_ssl_verification > 0, request_timeout as u64);
 
       EXIT_SUCCESS
     } {
@@ -251,6 +235,58 @@ ffi_fn! {
     }
 }
 
+ffi_fn! {
+  /// Set the options used when publishing verification results to the Pact Broker
+  ///
+  /// # Args
+  /// 
+  /// - `handle` - The pact verifier handle to update
+  /// - `provider_version` - Version of the provider to publish
+  /// - `build_url` - URL to the build which ran the verification
+  /// - `provider_tags` - Collection of tags for the provider
+  /// - `provider_tags_len` - Number of provider tags supplied
+  /// - `provider_branch` - Name of the branch used for verification
+  ///
+  /// # Safety
+  ///
+  /// All string fields must contain valid UTF-8. Invalid UTF-8
+  /// will be replaced with U+FFFD REPLACEMENT CHARACTER.
+  ///
+  fn pactffi_verifier_set_publish_options(
+    handle: *mut handle::VerifierHandle,
+    provider_version: *const c_char,
+    build_url: *const c_char,
+    provider_tags: *const *const c_char,
+    provider_tags_len: c_ushort,
+    provider_branch: *const c_char
+  ) -> c_int {
+    let handle = as_mut!(handle);
+    let provider_version = safe_str!(provider_version);
+    let build_url = if_null(build_url, "");
+    let provider_branch = if_null(provider_branch, "");
+
+    let build_url = if !build_url.is_empty() {
+      Some(build_url)
+    } else {
+      None
+    };
+
+    let tags = get_vector(provider_tags, provider_tags_len);
+
+    let branch = if !provider_branch.is_empty() {
+      Some(provider_branch)
+    } else {
+      None
+    };
+
+    handle.update_publish_options(provider_version, build_url, tags, branch);
+
+    EXIT_SUCCESS
+  } {
+    EXIT_FAILURE
+  }
+}
+
 ffi_fn! {
     /// Set the consumer filters for the Pact verifier.
     ///
@@ -366,14 +402,12 @@ ffi_fn! {
     fn pactffi_verifier_broker_source(
       handle: *mut handle::VerifierHandle,
       url: *const c_char,
-      provider_name: *const c_char,
       username: *const c_char,
       password: *const c_char,
       token: *const c_char
     ) {
       let handle = as_mut!(handle);
       let url = safe_str!(url);
-      let provider_name = safe_str!(provider_name);
       let username = if_null(username, "");
       let password = if_null(password, "");
       let token = if_null(token, "");
@@ -390,7 +424,7 @@ ffi_fn! {
         HttpAuth::None
       };
 
-      handle.add_pact_broker_source(url, provider_name, false, None, vec![], None, vec![], &auth);
+      handle.add_pact_broker_source(url, false, None, vec![], None, vec![], &auth);
     }
 }
 
@@ -417,7 +451,6 @@ ffi_fn! {
     fn pactffi_verifier_broker_source_with_selectors(
       handle: *mut handle::VerifierHandle,
       url: *const c_char,
-      provider_name: *const c_char,
       username: *const c_char,
       password: *const c_char,
       token: *const c_char,
@@ -433,7 +466,6 @@ ffi_fn! {
     ) {
       let handle = as_mut!(handle);
       let url = safe_str!(url);
-      let provider_name = safe_str!(provider_name);
       let provider_branch: Option<String> = if provider_branch.is_null() {
         None
       } else {
@@ -475,7 +507,7 @@ ffi_fn! {
         vec![]
       };
 
-      handle.add_pact_broker_source(url, provider_name, enable_pending > 0, wip, provider_tags_vector, provider_branch, selectors, &auth);
+      handle.add_pact_broker_source(url, enable_pending > 0, wip, provider_tags_vector, provider_branch, selectors, &auth);
     }
 }
 
@@ -721,3 +753,31 @@ ffi_fn! {
       std::ptr::null()
     }
 }
+
+ffi_fn! {
+    /// Extracts the standard output for the verification run. The returned string will need to be
+    /// freed with the `free_string` function call to avoid leaking memory.
+    ///
+    /// Will return a NULL pointer if the handle is invalid.
+    fn pactffi_verifier_output(handle: *const handle::VerifierHandle) -> *const c_char {
+      let handle = as_ref!(handle);
+      let output = CString::new(handle.output()).unwrap();
+      output.into_raw() as *const c_char
+    } {
+      std::ptr::null()
+    }
+}
+
+ffi_fn! {
+    /// Extracts the verification result as a JSON document. The returned string will need to be
+    /// freed with the `free_string` function call to avoid leaking memory.
+    ///
+    /// Will return a NULL pointer if the handle is invalid.
+    fn pactffi_verifier_json(handle: *const handle::VerifierHandle) -> *const c_char {
+      let handle = as_ref!(handle);
+      let output = CString::new(handle.json()).unwrap();
+      output.into_raw() as *const c_char
+    } {
+      std::ptr::null()
+    }
+}
